<?php
/*
Plugin Name: DrGlenn Quick Check
Description: Quickly checks installed plugins against the National Vulnerability Database (NVD)
Version: 1.9
Author: DrGlenn
*/

// Hook into the WordPress admin menu
add_action('admin_menu', 'drglenn_quick_check_menu');

function drglenn_quick_check_menu() {
    add_menu_page(
        'DrGlenn Quick Check',
        'DrGlenn Quick Check',
        'manage_options',
        'drglenn-quick-check',
        'drglenn_quick_check_page'
    );
}

function drglenn_quick_check_page() {
    echo '<h1>DrGlenn Quick Check</h1>';
    echo '<p>This tool will quickly check your installed plugins against the National Vulnerability Database (NVD).</p>';
    echo '<p>Click the button below to start the scan. Please note that this may take a minute to complete.</p>';

    echo '<button id="drglenn-start-scan" class="button button-primary">Start Scan</button>';
    echo '<div id="drglenn-scan-status" style="margin-top: 20px;"></div>';

    echo '<style>
        #drglenn-scan-status table {
            width: 100%;
            border-collapse: collapse;
        }
        #drglenn-scan-status th, #drglenn-scan-status td {
            padding: 10px;
            border: 1px solid #ddd;
            text-align: left;
        }
        #drglenn-scan-status tr:nth-child(even) {
            background-color: #f9f9f9;
        }
        .status-vulnerabilities {
            color: red;
            font-weight: bold;
        }
        .status-safe {
            color: green;
            font-weight: bold;
        }
        .status-checking {
            color: orange;
        }
    </style>';

    $plugins = json_encode(get_plugins());
    echo '<script>
        document.getElementById("drglenn-start-scan").addEventListener("click", function() {
            var status = document.getElementById("drglenn-scan-status");
            status.innerHTML = "<p>Stand by... This may take a minute.</p>";
            setTimeout(function() {
                drglenn_run_scan();
            }, 1000);
        });

        function drglenn_run_scan() {
            var plugins = ' . $plugins . ';
            var output = "<table class=\"wp-list-table widefat fixed striped\">" +
                "<thead><tr><th>Plugin Name</th><th>Version</th><th>Status</th></tr></thead><tbody>";

            Object.keys(plugins).forEach(function(file, index) {
                var name = plugins[file].Name;
                var version = plugins[file].Version;
                var rowId = "plugin-row-" + index;
                output += "<tr id=\"" + rowId + "\">" +
                    "<td>" + name + "</td>" +
                    "<td>" + version + "</td>" +
                    "<td class=\"status-checking\">Checking...</td>" +
                "</tr>";

                var searchUrl = "admin-ajax.php?action=drglenn_proxy_request&keyword=" + encodeURIComponent(name);
                console.log("Checking URL:", searchUrl);
                fetch(searchUrl)
                    .then(function(response) {
                        if (!response.ok) {
                            console.error("Network error:", response.statusText);
                            throw new Error("Network response was not ok: " + response.statusText);
                        }
                        return response.json();
                    })
                    .then(function(data) {
                        var statusCell = document.querySelector("#" + rowId + " td:last-child");
                        if (data.totalResults > 0) {
                            statusCell.textContent = "Vulnerabilities Found";
                            statusCell.className = "status-vulnerabilities";
                        } else {
                            statusCell.textContent = "No Vulnerabilities";
                            statusCell.className = "status-safe";
                        }
                    })
                    .catch(function(error) {
                        console.error("Fetch error:", error);
                        var statusCell = document.querySelector("#" + rowId + " td:last-child");
                        statusCell.textContent = "Error Checking: " + error.message;
                        statusCell.className = "status-checking";
                    });
            });

            output += "</tbody></table>";
            document.getElementById("drglenn-scan-status").innerHTML = output;
        }
    </script>';
}

// Create a server-side proxy to handle the NVD API request
add_action('wp_ajax_drglenn_proxy_request', 'drglenn_proxy_request');
function drglenn_proxy_request() {
    $keyword = sanitize_text_field($_GET['keyword']);
    $api_url = "https://services.nvd.nist.gov/rest/json/cves/1.0?keyword=" . urlencode($keyword);
    $response = wp_remote_get($api_url);

    if (is_wp_error($response)) {
        wp_send_json_error($response->get_error_message());
    } else {
        wp_send_json_success(json_decode(wp_remote_retrieve_body($response)));
    }

    wp_die();
}
